@@ -217,6 +217,12 @@ class CloudyModel(object):
                 self.r_out = self.radius_full + self.dr_full/2   
             self.r_in_cut = self.r_in
             self.r_out_cut = self.r_out
+            if self.Phi0 == 0.:
+                self.Phi = self.Q / (4 * np.pi * self.r_in**2)
+                self.Phi0 = self.Phi.sum()
+
+            
+
     ## 
     # @var r_in 
     # Initial radius [float] (cm)
@@ -409,6 +415,8 @@ class CloudyModel(object):
                 self.out['###Last'] = line
             elif 'Calculation stopped' in line:
                 self.out['stop'] = line
+            elif 'Cloudy ends' in line:
+                self.out['Cloudy ends'] = line
             elif 'Gas Phase Chemical Composition' in line:
                 for i in range(4):
                     self.out['Chem' + str(i + 1)] = file_.next()
@@ -482,22 +490,20 @@ class CloudyModel(object):
             self.Q[2] = float(pc.sextract(self.out['SED2'], 'Q(4.0-20):', 'Q(20--):'))
             self.Q[3] = float(pc.sextract(self.out['SED2'], 'Q(20--):', 'Ion pht'))
             self.Q = pow(10., self.Q)
-            self.Q0 = self.Q.sum()
             self.plan_par = False
         except:
-#            self.Q = None
-            self.Q0 = None
+            pass
+        self.Q0 = self.Q.sum()
         try:
             self.Phi[0] = float(pc.sextract(self.out['SED2'], 'phi(1.0-1.8):', 'phi(1.8-4.0):'))
             self.Phi[1] = float(pc.sextract(self.out['SED2'], 'phi(1.8-4.0):', 'phi(4.0-20):'))
             self.Phi[2] = float(pc.sextract(self.out['SED2'], 'phi(4.0-20):', 'phi(20--):'))
             self.Phi[3] = float(pc.sextract(self.out['SED2'], 'phi(20--):', 'Ion pht'))
             self.Phi = pow(10., self.Phi)
-            self.Phi0 = self.Phi.sum()
             self.plan_par = True
         except:
-#            self.Q = None
-            self.Phi0 = None
+            pass
+        self.Phi0 = self.Phi.sum()
         
         self.abund = {}
         Chem = self.out['Chem1'][0:-1]
@@ -669,16 +675,54 @@ class CloudyModel(object):
     def heat(self):
         """ array of heating (on r_range)"""
         return self.heat_full[self.r_range]
+    
+    def _quiet_div(self, a, b):
+        if a is None or b is None:
+            to_return = None
+        else:
+            np.seterr(all="ignore")
+            to_return = a / b
+            np.seterr(all=None)
+        return to_return
 
-    ## log(U) in each zone [float array], with U(r) = \f$ Q_0 / (4.\pi.r^2.n_H.c)\f$
+    ##rad_integ(a) = \f$\int a.ff.dr\f$ 
+    def rad_integ(self, a):
+        """ integral of a on the radius"""
+        if a is None or self.dr is None:
+            return None
+        else:
+            #return np.trapz(a, self.radius)
+            return (a * self.drff).sum()
+    
+    ##vol_integ(a) = \f$\int a.ff.dV\f$
+    def vol_integ(self, a):
+        """ integral of a on the volume"""
+        if a is None or self.dv is None:
+            return None
+        else:
+            return (a * self.dvff).sum()
+            #return 4.0*np.pi*np.trapz((a*self.radius**2*ff)[self.r_range], self.radius[self.r_range])
+
+    ##vol_mean(a, b) = \f$\frac{\int a.b.ff.dV}{\int b.ff.dV}\f$
+    def vol_mean(self, a, b = 1.):
+        """ Return the mean value of a weighted by b on the volume"""
+        return self._quiet_div(self.vol_integ(a * b), self.vol_integ(b))
+    
+    ##rad_mean(a, b) = \f$\frac{\int a.b.dr}{\int b.dr}\f$    
+    def rad_mean(self, a, b = 1.):
+        """ Return the mean value of a weighted by b on the radius"""
+        return self._quiet_div(self.rad_integ(a * b), self.rad_integ(b))    
+
+    ## log(U) in each zone [float array], with U(r) = \f$ Phi_0 * (r_0/r)^2/ (n_H.c)\f$
     @property
     def log_U(self):
-        """ U = Q0 /(4 pi radius**2 nH c)"""
+        """ U = Phi0 * (r_in/rarius) / (nH c)"""
         try:
-            return np.log10(self.Q0 / (4. * np.pi * self.radius ** 2 * self.nH * pc.CST.CLIGHT))
+            log_U = np.log10(self.Phi0 * (self.r_in/self.radius)**2 / (self.nH * pc.CST.CLIGHT))
         except:
             self.log_.warn('No U computed', calling = self.calling)
-            return None
+            log_U = None
+        return log_U
         
     ## log_U_mean = \f$\frac{\int U.dV}{\int dV}\f$ [float]
     @property
@@ -692,9 +736,9 @@ class CloudyModel(object):
     ## log_U_mean_ne = \f$\frac{\int U.ne.dV}{\int ne.dV}\f$ [float]
     @property
     def log_U_mean_ne(self):
-        """ log of mean value of U on the volume """
+        """ log of mean value of U on the volume weighted by ne.nH"""
         if self.log_U is not None:
-            return np.log10(self.vol_mean(self.ne*10**self.log_U, self.ne))
+            return np.log10(self.vol_mean(10**self.log_U, self.nenH))
         else:
             return None      
     
@@ -1239,43 +1283,6 @@ class CloudyModel(object):
             G0 = abs(np.trapz(y = self.get_cont_y('ntrans', 'ec3A', dist_norm = dist_norm)[lam_range], x = lam[lam_range]))/norm
         return G0
 
-    def _quiet_div(self, a, b):
-        if a is None or b is None:
-            to_return = None
-        else:
-            np.seterr(all="ignore")
-            to_return = a / b
-            np.seterr(all=None)
-        return to_return
-
-    ##rad_integ(a) = \f$\int a.ff.dr\f$ 
-    def rad_integ(self, a):
-        """ integral of a on the radius"""
-        if a is None or self.dr is None:
-            return None
-        else:
-            #return np.trapz(a, self.radius)
-            return (a * self.drff).sum()
-    
-    ##vol_integ(a) = \f$\int a.ff.dV\f$
-    def vol_integ(self, a):
-        """ integral of a on the volume"""
-        if a is None or self.dv is None:
-            return None
-        else:
-            return (a * self.dvff).sum()
-            #return 4.0*np.pi*np.trapz((a*self.radius**2*ff)[self.r_range], self.radius[self.r_range])
-
-    ##vol_mean(a, b) = \f$\frac{\int a.b.ff.dV}{\int b.ff.dV}\f$
-    def vol_mean(self, a, b = 1.):
-        """ Return the mean value of a weighted by b on the volume"""
-        return self._quiet_div(self.vol_integ(a * b), self.vol_integ(b))
-    
-    ##rad_mean(a, b) = \f$\frac{\int a.b.dr}{\int b.dr}\f$    
-    def rad_mean(self, a, b = 1.):
-        """ Return the mean value of a weighted by b on the radius"""
-        return self._quiet_div(self.rad_integ(a * b), self.rad_integ(b))    
-
     def _get_r_out_cut(self):    
         return self.__r_out_cut
     
@@ -1869,7 +1876,7 @@ class CloudyInput(object):
                 else:
                     self.log_.warn('unkown symbol : {0}'.format(sym), calling = self.calling)
         elif elem is not None:
-            self._abund[elem] = value
+            self._abund[SYM2ELEM[elem]] = value
         elif predef is not None:
             self._abund_predef = predef
         self._metals = metals
@@ -2058,7 +2065,7 @@ class CloudyInput(object):
             self.log_.message('Input writen in {0}'.format(file_name), calling = self.calling)
             f.close()
             
-    def run_cloudy(self, dir_ = None, n_proc = 1, use_make = False, model_name = None):
+    def run_cloudy(self, dir_ = None, n_proc = 1, use_make = False, model_name = None, precom=""):
         """
         Method to run cloudy.
         Parameters:
@@ -2068,11 +2075,11 @@ class CloudyInput(object):
                 assuming that model_name.in exists
             - model_name:  if None, the models of this object is run, 
                 if not None, used by: make name="model_name" or cloudy < model_name.in
-        
+            - precom: a string to put before Cloudy (e.g. "\nice 10")
         """
         if model_name is None:
             model_name = self.model_name
-        run_cloudy(dir_ = dir_, n_proc = n_proc, use_make = use_make, model_name = model_name)
+        run_cloudy(dir_ = dir_, n_proc = n_proc, use_make = use_make, model_name = model_name, precom=precom)
     
     def print_make_file(self, dir_ = None):
         """
@@ -2111,7 +2118,7 @@ all: $(OBJ)
     makefile.close()
 
 ## Function used to run Cloudy on input files.                
-def run_cloudy(dir_ = None, n_proc = 1, use_make = True, model_name = None):
+def run_cloudy(dir_ = None, n_proc = 1, use_make = True, model_name = None, precom=""):
     """
     Run a (set of ) cloudy model(s)
     
@@ -2122,6 +2129,7 @@ def run_cloudy(dir_ = None, n_proc = 1, use_make = True, model_name = None):
             assuming that model_name.in exists
         - model_name:  if not None, used by: make name="model_name" or cloudy < model_name.in
             if None and use_make, make will run any pending model
+        - precom: a string to put before Cloudy (e.g. "\nice 10")
     """
     if dir_ is None:
         dir_ = '/'.join(model_name.split('/')[0:-1])
@@ -2135,7 +2143,7 @@ def run_cloudy(dir_ = None, n_proc = 1, use_make = True, model_name = None):
         if model_name is None:
             pc.log_.error('Model name must be set', calling = 'run_cloudy')
         else:
-            to_run = 'cd {0} ; {1}'.format(dir_, pc.config.cloudy_exe)
+            to_run = 'cd {0} ; {1} {2}'.format(dir_, precom, pc.config.cloudy_exe)
             stdin = file('{0}/{1}.in'.format(dir_, model_name.split('/')[-1]), 'r')
             stdout = file('{0}/{1}.out'.format(dir_, model_name.split('/')[-1]), 'w')   
     pc.log_.message('running: {0}'.format(to_run), calling = 'run_cloudy')
